@@ -2,36 +2,35 @@
 
 push @ARGV, "";
 
-if($ARGV[0] eq "--help")
+if ($ARGV[0] eq "--help")
 {
 print STDERR <<EOT;
-usage:	ucstoany.pl [-f][+u|-u]|[+f[+g|-g]] [+o|-o OUTPUT]
+usage:	ucstoany.pl [-f [+u|-u]]|[+f [+g|-g]] [+o|-o NAME] [+b]
 	[+[CHAR]] [--] INPUT REGISTRY ENCODING [TABLE...]
 
--f	Filter mode - don't output characters with unicode
-	FFFF. This is the default.
-  +u	Unicode mode - encode characters with index >= 32 with
-	their unicodes. Default for ISO10646-1 output.
-  -u	Index mode -  encode the characters with their indexes
-	(FFFF counts). Default for any other output.
+-f	Filter mode - discard characters with unicode FFFF.
+	This is the default.
+  +u	Encode characters with index >= 32 with their unicodes.
+	Default for ISO10646-1 output.
+  -u	Encode all characters with their indexes (FFFF counts).
+	Default for any other output.
 
-+f	Fillout mode - encode with unicodes, output characters
++f	Fillout mode - encode with unicodes, emit characters
 	with unicode FFFF as the default character.
   +g	Exchange the characters in range 00...1F with these at
-        C0...DF. Default for 256 character fonts not starting
-	with FFFD.
+	C0...DF. Default for 8-pixel wide fonts with 256...512
+	characters starting with 00A3.
   -g	Do not exchange. Default for all other fonts.
 
 +o	Output to INPUT-REGISTRY-ENCODING (using the same name
 	as ucs2any, but preserving the INPUT directory).
 -o NAME	Output to NAME.
 
-+b	Use binary mode for output. Only affects the systems
-	that distinguish between text and binary files.
--b	Use text mode for output. This is the default.
++b	Use binary mode for output (the default is text mode).
+	No effect on POSIX systems.
 
-+[CHAR]	Set the default character to CHAR. If no CHAR is
-        specified, the one from INPUT is used.
++[CHAR]	Set the default character to CHAR (decimal value). If
+	no CHAR is specified, the one from INPUT is used.
 
 --	Terminate the option list.
 
@@ -49,16 +48,14 @@ unicode and fillout modes, 46 (period) for index mode.
 
 Any options not specified in the above order are treated as
 non-option arguments.
-
-Warning: this program is not compatible with ucs2any.pl!
 EOT
 	exit 0;
 }
 
-if($ARGV[0] eq "--version")
+if ($ARGV[0] eq "--version")
 {
 print STDERR <<EOT;
-ucstoany.pl 0.0.4, Copyright (C) 2003 Dimitar Zhekov
+ucstoany.pl 0.1.1, Copyright (C) 2008 Dimitar Toshkov Zhekov
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License as
@@ -70,24 +67,22 @@ but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
-This program is not based on ucs2any.pl or any other program.
-
 Report bugs to jimmy\@is-vn.bg
 EOT
 	exit 0;
 }
 
-if($ARGV[0] eq "+f")
+if ($ARGV[0] eq "+f")
 {
 	$control = 0;
 	shift @ARGV;
 
-	if($ARGV[0] eq "+g")
+	if ($ARGV[0] eq "+g")
 	{
 		$exchange = 1;
 		shift @ARGV;
 	}
-	elsif($ARGV[0] eq "-g")
+	elsif ($ARGV[0] eq "-g")
 	{
 		$exchange = 0;
 		shift @ARGV;
@@ -98,33 +93,37 @@ else
 	$exchange = 0;
 	if($ARGV[0] eq "-f") { shift @ARGV; }
 
-	if($ARGV[0] eq "+u")
+	if ($ARGV[0] eq "+u")
 	{
 		$control = 32;
 		shift @ARGV;
 	}
-	elsif($ARGV[0] eq "-u")
+	elsif ($ARGV[0] eq "-u")
 	{
 		$control = 65536;
 		shift @ARGV;
 	}
 }
 
-if($ARGV[0] eq "+o")
+if ($ARGV[0] eq "+o")
 {
 	$output = "";
 	shift @ARGV;
 }
-elsif($ARGV[0] eq "-o")
+elsif ($ARGV[0] eq "-o")
 {
 	shift @ARGV;
-
-	$ARGV[0] ne "" || die("$0: -o requires a non-empty argument\n");
+	$#ARGV > 0 || die("$0: -o requires an argument\n");
 	$output = $ARGV[0];
 	shift @ARGV;
 }
+elsif ($ARGV[0] =~ /^-o(.*)$/)
+{
+	$output = $1;
+	shift @ARGV;
+}
 
-if($ARGV[0] eq "+b")
+if ($ARGV[0] eq "+b")
 {
 	$binary = 1;
 	shift @ARGV;
@@ -132,68 +131,70 @@ if($ARGV[0] eq "+b")
 else
 {
 	$binary = 0;
-	if($ARGV[0] eq "-b") { shift @ARGV; }
+	if ($ARGV[0] eq "-b") { shift @ARGV; }
 }
 
-$_ = $ARGV[0];
-if(/\+(.*)/)
+if ($ARGV[0] =~ /\+\s*([0-9]*)\s*$/)
 {
 	$default = $1;
 	shift @ARGV;
 }
 
-if($ARGV[0] eq '--') { shift @ARGV; }
+if ($ARGV[0] eq '--') { shift @ARGV; }
+elsif ($ARGV[0] =~ /^([-+][0-9a-z])$/) { print STDERR "$0: suspicuous $1, use -- to terminate the option list\n"; }
 pop @ARGV;
 
-if($#ARGV == 2) { $ARGV[3] = "-"; }
-elsif($#ARGV < 2) { die("$0: invalid arguments, try --help\n"); }
+if ($#ARGV == 2) { $ARGV[3] = "-"; }
+elsif ($#ARGV < 2) { die("$0: invalid arguments, try --help\n"); }
 
 open(BDF, "<$ARGV[0]") || die("$0: $ARGV[0]: $!\n");
 
-while(<BDF>)
+while (<BDF>)
 {
 	$header .= $_;
 	last if /^CHARS\s/;
 }
 
-while(<BDF>)
+while (<BDF>)
 {
-	if(/^STARTCHAR\s+(.+)$/) { $startchar = $1; }
-	elsif(/^ENCODING\s+(.+)$/) { $encoding = $1; }
+	if (/^STARTCHAR\s+(.+)$/) { $startchar = $1; }
+	elsif (/^ENCODING\s+(.+)$/) { $encoding = $1; }
 	$buffer .= $_;
-	if(/^ENDCHAR/)
+	if (/^ENDCHAR$/)
 	{
 		$startchar ne "" || die("$0: $ARGV[0]: ENDCHAR without STARTCHAR\n");
 		$encoding ne "" || die("$0: $ARGV[0]: no ENCODING for $startchar\n");
 		$bitmap{$encoding} = $buffer;
 		$buffer = $startchar = $encoding = "";
 	}
+	elsif (!defined($exchange) && /^BBX\s+([0-9]+)/ && $1 != 8) { $exchange = 0; }
 }
 
 close BDF;
 
-$charset = $_ = "-$ARGV[1]-$ARGV[2]";
-if(!defined($control)) { $control = /^-iso10646-1$/i ? 32 : 65536; }
+$charset = "-$ARGV[1]-$ARGV[2]";
+if (!defined($control)) { $control = ($charset =~ /^-iso10646-1$/i) ? 32 : 65536; }
 
 $chars = 0;
-for($index = 3; $index <= $#ARGV; $index++)
+for ($index = 3; $index <= $#ARGV; $index++)
 {
 	open(UNI, "<$ARGV[$index]") || die("$0: $ARGV[$index]: $!\n");
-	while(<UNI>)
+	while (<UNI>)
 	{
 		next if /^\s*$/;
 		/^([0-9a-fA-F]{1,4})$/ || die("$0: $ARGV[$index]: invalid unicode $_\n");
 		push @unimap, hex($1);
-		if(!$control || hex($1) != 65535) { $chars++; }
+		if (!$control || hex($1) != 65535) { $chars++; }
 	}
 	close UNI;
 }
+$chars > 0 || die("$0: empty unicode table(s)\n");
 
-if(!defined($exchange)) { $exchange = $chars == 256 && $unimap[0] != 65533; }
-if($exchange)
+if (!defined($exchange)) { $exchange = $chars >= 256 && $chars <= 512 && $unimap[0] != 65533; }
+if ($exchange)
 {
 	@unimap >= 0xE0 || die("$0: not enough characters for exchange\n");
-	for($index = 0x00; $index < 0x20; $index++)
+	for ($index = 0x00; $index < 0x20; $index++)
 	{
 		$_ = $unimap[$index];
 		$unimap[$index] = $unimap[$index + 0xC0];
@@ -201,16 +202,15 @@ if($exchange)
 	}
 }
 
-if(!defined($default)) { $default = $control == 65536 ? 46 : 65533; }
-elsif($default eq "")
+if (!defined($default)) { $default = $control == 65536 ? 46 : 65533; }
+elsif ($default eq "")
 {
-	$_ = $header;
-	/^DEFAULT_CHAR\s(.+)$/m || die("$0: $ARGV[0]: unable to obtain DEFAULT_CHAR\n");
+	$header =~ /^DEFAULT_CHAR\s(.+)$/m || die("$0: $ARGV[0]: unable to obtain DEFAULT_CHAR\n");
 	$default = $1;
 }
 
-if(!defined($output)) { $output = "-"; }
-elsif($output eq "") { if($ARGV[0] =~ /^(.*).bdf$/) { $output = "$1$charset.bdf" ; } else { $output = "$ARGV[0]$charset"; } }
+if (!defined($output)) { $output = "-"; }
+elsif ($output eq "") { if ($ARGV[0] =~ /^(.*).bdf$/) { $output = "$1$charset.bdf" ; } else { $output = "$ARGV[0]$charset"; } }
 
 $header =~ s/^(FONT\s.*)-.*-.*$/$1$charset/m || die("$0: $output: unable to change FONT registry-encoding\n");
 $header =~ s/^(CHARSET_REGISTRY\s).*$/$1"$ARGV[1]"/m || die("$0: $output: unable to change CHARSET_REGISTRY\n");
@@ -218,18 +218,28 @@ $header =~ s/^(CHARSET_ENCODING\s).*$/$1"$ARGV[2]"/m || die("$0: $output: unable
 $header =~ s/^(DEFAULT_CHAR\s).*$/$1$default/m || die("$0: $output: unable to change DEFAULT_CHAR\n");
 $header =~ s/^(CHARS\s).*$/$1$chars/m || die("$0: $output: unable to change CHARS\n");
 
+sub int { $int = $!; }
+sub bye
+{
+	close OUT;
+	$output ne "-" && unlink($output) != 1 && print STDERR "$0: $output: $!\n";
+	die("@_");
+}
+
 open(OUT, ">$output") || die("$0: $output: $!\n");
-if($binary) { binmode(OUT) || die("$0: $output: $!\n"); }
+$SIG{INT} = 'int';
+if ($binary) { binmode(OUT) || bye("$0: $output: $!\n"); }
 print OUT $header;
 
-for($index = 0; $index < @unimap; $index++)
+for ($index = 0; $index < @unimap; $index++)
 {
 	$_ = $unimap[$index];
 	$encoding = $index >= $control ? $_ : $index;
-	if($_ == 65535) { if($control) { next; } else { $_ = $default ; } }
-	$bitmap{$_} =~ s/^(ENCODING\s).*$/$1$encoding/m || die("$0: $output: unable to change encoding for $_\n");
+	if ($_ == 65535) { if ($control) { next; } else { $_ = $default ; } }
+	$bitmap{$_} =~ s/^(ENCODING\s).*$/$1$encoding/m || bye("$0: $output: unable to change encoding for $_\n");
 	print OUT $bitmap{$_};
 }
 
 print OUT $buffer;
-close OUT;
+close OUT || bye("$0: $output: $!\n");
+defined($int) && bye("$0: $int\n");
